#include "config.h"

#ifndef NO_SSL

# include <openssl/ssl.h>
# include <openssl/rand.h>
# include <openssl/err.h>
# include <openssl/rc4.h>
# include <openssl/bn.h>
# include <openssl/hmac.h>

# include "sslwrapper.h"

#if OPENSSL_VERSION_NUMBER >= 0x00908000L
# define OPENSSL_0_9_8    
#else
# undef OPENSSL_0_9_8
#endif

#pragma warning( disable : 4312 )

typedef int (far __cdecl *DLL_FUNC)();

typedef struct {
    const char *name;
    DLL_FUNC func;
} ssl_func_t;

static ssl_func_t lib_func[] = {
                                    { "RAND_load_file", NULL },
#define INDEX_RAND_load_file 0
                                    { "RAND_status", NULL },
#define INDEX_RAND_status 1
                                    { "RAND_egd", NULL },
#define INDEX_RAND_egd 2
                                    { "RAND_file_name", NULL },
#define INDEX_RAND_file_name 3
                                    { "RAND_write_file", NULL },
#define INDEX_RAND_write_file 4
                                    { "BIO_sock_should_retry", NULL },
#define INDEX_BIO_sock_should_retry 5
                                    { "ERR_error_string", NULL },
#define INDEX_ERR_error_string 6
                                    { "ERR_print_errors_fp", NULL },
#define INDEX_ERR_print_errors_fp 7
                                    { "ASN1_INTEGER_get", NULL },
#define INDEX_ASN1_INTEGER_get 8
                                    { "ASN1_INTEGER_cmp", NULL },
#define INDEX_ASN1_INTEGER_cmp 9
                                    { "sk_value", NULL },
#define INDEX_sk_value 10
                                    { "sk_num", NULL },
#define INDEX_sk_num 11
                                    { "X509_NAME_oneline", NULL },
#define INDEX_X509_NAME_oneline 12
                                    { "X509_get_serialNumber", NULL },
#define INDEX_X509_get_serialNumber 13
                                    { "X509_cmp_current_time", NULL },
#define INDEX_X509_cmp_current_time 14
                                    { "X509_OBJECT_free_contents", NULL },
#define INDEX_X509_OBJECT_free_contents 15
                                    { "X509_STORE_CTX_set_error", NULL },
#define INDEX_X509_STORE_CTX_set_error 16
                                    { "X509_CRL_verify", NULL },
#define INDEX_X509_CRL_verify 17
                                    { "X509_get_pubkey", NULL },
#define INDEX_X509_get_pubkey 18
                                    { "X509_STORE_CTX_cleanup", NULL },
#define INDEX_X509_STORE_CTX_cleanup 19
                                    { "X509_STORE_get_by_subject", NULL },
#define INDEX_X509_STORE_get_by_subject 20
                                    { "X509_STORE_CTX_init", NULL },
#define INDEX_X509_STORE_CTX_init 21
                                    { "X509_get_issuer_name", NULL },
#define INDEX_X509_get_issuer_name 22
                                    { "X509_get_subject_name", NULL },
#define INDEX_X509_get_subject_name 23
                                    { "X509_STORE_CTX_get_current_cert", NULL },
#define INDEX_X509_STORE_CTX_get_current_cert 24
				    { "X509_STORE_CTX_get_ex_data", NULL },
#define INDEX_X509_STORE_CTX_get_ex_data 25
				    { "RC4", NULL },
#define INDEX_RC4 26
				    { "RC4_set_key", NULL },
#define INDEX_RC4_set_key 27
                                    { "ERR_get_error", NULL },
#define INDEX_ERR_get_error 28
				    { "RAND_bytes", NULL },
#define INDEX_RAND_bytes 29
				    { "BN_cmp", NULL },
#define INDEX_BN_cmp 30
				    { "BN_num_bits", NULL },
#define INDEX_BN_num_bits 31
				    { "BN_is_bit_set", NULL },
#define INDEX_BN_is_bit_set 32
				    { "BN_rand", NULL },
#define INDEX_BN_rand 33
				    { "BN_new", NULL },
#define INDEX_BN_new 34
				    { "BN_clear_free", NULL },
#define INDEX_BN_clear_free 35
				    { "BN_hex2bn", NULL },
#define INDEX_BN_hex2bn 36
				    { "BN_free", NULL },
#define INDEX_BN_free 37
				    { "BN_bin2bn", NULL },
#define INDEX_BN_bin2bn 38
				    { "BN_bn2bin", NULL },
#define INDEX_BN_bn2bin 39
				    { "DH_generate_key", NULL },
#define INDEX_DH_generate_key 40
				    { "DH_new", NULL },
#define INDEX_DH_new 41
				    { "DH_free", NULL },
#define INDEX_DH_free 42
				    { "DH_compute_key", NULL },
#define INDEX_DH_compute_key 43
				    { "DH_size", NULL },
#define INDEX_DH_size 44
				    { "HMAC_Init", NULL },
#define INDEX_HMAC_Init 45
				    { "HMAC_Update", NULL },
#define INDEX_HMAC_Update 46
				    { "HMAC_Final", NULL },
#define INDEX_HMAC_Final 47
				    { "HMAC_CTX_cleanup", NULL },
#define INDEX_HMAC_CTX_cleanup 48
				    { "EVP_md5", NULL },
#define INDEX_EVP_md5 49
				    { "EVP_ripemd160", NULL },
#define INDEX_EVP_ripemd160 50
				    { "EVP_sha1", NULL },
#define INDEX_EVP_sha1 51
				    { "EVP_DigestUpdate", NULL },
#define INDEX_EVP_DigestUpdate 52
				    { "EVP_DigestInit", NULL },
#define INDEX_EVP_DigestInit 53
				    { "EVP_DigestFinal", NULL },
#define INDEX_EVP_DigestFinal 54
				    { "EVP_CIPHER_CTX_set_key_length", NULL },
#define INDEX_EVP_CIPHER_CTX_set_key_length 55
				    { "EVP_CipherInit", NULL },
#define INDEX_EVP_CipherInit 56
				    { "EVP_aes_128_cbc", NULL },
#define INDEX_EVP_aes_128_cbc 57
				    { "EVP_bf_cbc", NULL },
#define INDEX_EVP_bf_cbc 58
				    { "EVP_des_ede3_cbc", NULL },
#define INDEX_EVP_des_ede3_cbc 59
				    { "EVP_CIPHER_CTX_init", NULL },
#define INDEX_EVP_CIPHER_CTX_init 60
				    { "EVP_CIPHER_CTX_cleanup", NULL },
#define INDEX_EVP_CIPHER_CTX_cleanup 61
				    { "RSA_public_decrypt", NULL },
#define INDEX_RSA_public_decrypt 62
				    { "RSA_size", NULL },
#define INDEX_RSA_size 63
				    { "RSA_new", NULL },
#define INDEX_RSA_new 64
				    { "RSA_free", NULL },
#define INDEX_RSA_free 65
				    { "DSA_new", NULL },
#define INDEX_DSA_new 66
				    { "DSA_free", NULL },
#define INDEX_DSA_free 67
				    { "DSA_SIG_new", NULL },
#define INDEX_DSA_SIG_new 68
				    { "DSA_SIG_free", NULL },
#define INDEX_DSA_SIG_free 69
				    { "DSA_do_verify", NULL },
#define INDEX_DSA_do_verify 70
                                    { NULL, NULL }
                                };

static ssl_func_t ssl_func[] = {
                                    { "SSL_CTX_set_default_verify_paths", NULL },
#define INDEX_SSL_CTX_set_default_verify_paths 0
                                    { "SSL_CTX_set_verify", NULL },
#define INDEX_SSL_CTX_set_verify 1
                                    { "SSL_CTX_ctrl", NULL },
#define INDEX_SSL_CTX_ctrl 2
                                    { "SSL_CTX_new", NULL },
#define INDEX_SSL_CTX_new 3
                                    { "SSLv23_client_method", NULL },
#define INDEX_SSLv23_client_method 4
                                    { "SSL_library_init", NULL },
#define INDEX_SSL_library_init 5
                                    { "SSL_load_error_strings", NULL },
#define INDEX_SSL_load_error_strings 6
                                    { "SSL_CTX_free", NULL },
#define INDEX_SSL_CTX_free 7
                                    { "SSL_free", NULL },
#define INDEX_SSL_free 8
                                    { "SSL_shutdown", NULL },
#define INDEX_SSL_shutdown 9
                                    { "SSL_read", NULL },
#define INDEX_SSL_read 10
                                    { "SSL_write", NULL },
#define INDEX_SSL_write 11
                                    { "SSL_want", NULL },
#define INDEX_SSL_want 12
                                    { "SSL_get_error", NULL },
#define INDEX_SSL_get_error 13
                                    { "SSL_connect", NULL },
#define INDEX_SSL_connect 14
                                    { "SSL_copy_session_id", NULL },
#define INDEX_SSL_copy_session_id 15
                                    { "SSL_ctrl", NULL },
#define INDEX_SSL_ctrl 16
                                    { "SSL_set_fd", NULL },
#define INDEX_SSL_set_fd 17
                                    { "SSL_new", NULL },
#define INDEX_SSL_new 18
                                    { "SSL_get_version", NULL },
#define INDEX_SSL_get_version 19
                                    { "SSL_set_ex_data", NULL },
#define INDEX_SSL_set_ex_data 20
                                    { "SSL_get_ex_new_index", NULL },
#define INDEX_SSL_get_ex_new_index 21
                                    { "SSL_get_ex_data", NULL },
#define INDEX_SSL_get_ex_data 22
				    { "SSL_get_ex_data_X509_STORE_CTX_idx", NULL },
#define INDEX_SSL_get_ex_data_X509_STORE_CTX_idx 23
                                    { NULL, NULL }
                                };

int RSA_public_decrypt(int flen, const unsigned char *from, 
		       unsigned char *to, RSA *rsa,int padding)
{
    return lib_func[INDEX_RSA_public_decrypt].func(flen, from, to, rsa, padding);
}

int RSA_size(const RSA *rsa)
{
    return lib_func[INDEX_RSA_size].func(rsa);
}

RSA *RSA_new(void)
{
    return (RSA *)lib_func[INDEX_RSA_new].func();
}

void RSA_free(RSA *r)
{
    lib_func[INDEX_RSA_free].func(r);
}

DSA *DSA_new(void)
{
    return (DSA *)lib_func[INDEX_DSA_new].func();
}

void DSA_free(DSA *r)
{
    lib_func[INDEX_DSA_free].func(r);
}

DSA_SIG *DSA_SIG_new(void)
{
    return (DSA_SIG *)lib_func[INDEX_DSA_SIG_new].func();
}

void DSA_SIG_free(DSA_SIG *a)
{
    lib_func[INDEX_DSA_SIG_free].func(a);
}

int DSA_do_verify(const unsigned char *dgst,int dgst_len,
                  DSA_SIG *sig,DSA *dsa)
{
    return lib_func[INDEX_DSA_do_verify].func(dgst, dgst_len, sig, dsa);
}

const EVP_MD *EVP_md5(void)
{
    return (EVP_MD *)lib_func[INDEX_EVP_md5].func();
}

const EVP_MD *EVP_ripemd160(void)
{
    return (EVP_MD *)lib_func[INDEX_EVP_ripemd160].func();
}

const EVP_MD *EVP_sha1(void)
{
    return (EVP_MD *)lib_func[INDEX_EVP_sha1].func();
}

int EVP_DigestUpdate(EVP_MD_CTX *ctx, const void *d, size_t cnt)
{
    return lib_func[INDEX_EVP_DigestUpdate].func(ctx, d, cnt);
}

int EVP_DigestInit(EVP_MD_CTX *ctx, const EVP_MD *type)
{
    return lib_func[INDEX_EVP_DigestInit].func(ctx, type);
}

int EVP_DigestFinal(EVP_MD_CTX *ctx,unsigned char *md,unsigned int *s)
{
    return lib_func[INDEX_EVP_DigestFinal].func(ctx, md, s);
}

int EVP_CIPHER_CTX_set_key_length(EVP_CIPHER_CTX *x, int keylen)
{
    return lib_func[INDEX_EVP_CIPHER_CTX_set_key_length].func(x, keylen);
}

 int EVP_CipherInit(EVP_CIPHER_CTX *ctx,const EVP_CIPHER *cipher,
		    const unsigned char *key,const unsigned char *iv,
		    int enc)
 {
     return lib_func[INDEX_EVP_CipherInit].func(ctx, cipher, key, iv, enc);
 }

 const EVP_CIPHER *EVP_aes_128_cbc(void)
 {
     return (const EVP_CIPHER *) lib_func[INDEX_EVP_aes_128_cbc].func();
 }

 const EVP_CIPHER *EVP_bf_cbc(void)
 {
     return (const EVP_CIPHER *) lib_func[INDEX_EVP_bf_cbc].func();
 }

 const EVP_CIPHER *EVP_des_ede3_cbc(void)
 {
     return (const EVP_CIPHER *) lib_func[INDEX_EVP_des_ede3_cbc].func();
 }

void EVP_CIPHER_CTX_init(EVP_CIPHER_CTX *a)
{
    lib_func[INDEX_EVP_CIPHER_CTX_init].func(a);
}

int EVP_CIPHER_CTX_cleanup(EVP_CIPHER_CTX *a)
{
    return lib_func[INDEX_EVP_CIPHER_CTX_cleanup].func(a);
}

void HMAC_Init(HMAC_CTX *ctx, const void *key, int len,
	       const EVP_MD *md)
{
    lib_func[INDEX_HMAC_Init].func(ctx, key, len, md);
}

void HMAC_Update(HMAC_CTX *ctx, const unsigned char *data, size_t len)
{
    lib_func[INDEX_HMAC_Update].func(ctx, data, len);
}

void HMAC_Final(HMAC_CTX *ctx, unsigned char *md, unsigned int *len)
{
    lib_func[INDEX_HMAC_Final].func(ctx, md, len);
}

void HMAC_CTX_cleanup(HMAC_CTX *ctx)
{
    lib_func[INDEX_HMAC_CTX_cleanup].func(ctx);
}
 
int DH_generate_key(DH *dh)
{
    return lib_func[INDEX_DH_generate_key].func(dh);
}

DH *DH_new(void)
{
    return (DH *)lib_func[INDEX_DH_new].func();
}

void DH_free(DH *dh)
{
    lib_func[INDEX_DH_free].func(dh);
}

int DH_compute_key(unsigned char *key,const BIGNUM *pub_key,DH *dh)
{
    return lib_func[INDEX_DH_compute_key].func(key, pub_key, dh);
}

int DH_size(const DH *dh)
{
    return lib_func[INDEX_DH_size].func(dh);
}

int BN_cmp(const BIGNUM *a, const BIGNUM *b)
{
    return lib_func[INDEX_BN_cmp].func(a, b);
}

int BN_num_bits(const BIGNUM *a)
{
    return lib_func[INDEX_BN_num_bits].func(a);
}

int BN_is_bit_set(const BIGNUM *a, int n)
{
    return lib_func[INDEX_BN_is_bit_set].func(a, n);
}

int BN_rand(BIGNUM *rnd, int bits, int top,int bottom)
{
    return lib_func[INDEX_BN_rand].func(rnd, bits, top, bottom);
}

BIGNUM *BN_new(void)
{
    return (BIGNUM *)lib_func[INDEX_BN_new].func();
}

void BN_clear_free(BIGNUM *a)
{
    lib_func[INDEX_BN_clear_free].func(a);
}

int BN_hex2bn(BIGNUM **a, const char *str)
{
    return lib_func[INDEX_BN_hex2bn].func(a, str);
}

void BN_free(BIGNUM *a)
{
    lib_func[INDEX_BN_free].func(a);
}

BIGNUM *BN_bin2bn(const unsigned char *s,int len,BIGNUM *ret)
{
    return (BIGNUM *)lib_func[INDEX_BN_bin2bn].func(s, len, ret);
}

int BN_bn2bin(const BIGNUM *a, unsigned char *to)
{
    return lib_func[INDEX_BN_bn2bin].func(a, to);
}

int RAND_bytes(unsigned char *buf, int num)
{
    return lib_func[INDEX_RAND_bytes].func(buf, num);
}

unsigned long ERR_get_error(void)
{
    return lib_func[INDEX_ERR_get_error].func();
}

void RC4_set_key(RC4_KEY *key, int len, const unsigned char *data)
{
    lib_func[INDEX_RC4_set_key].func(key, len, data);
}

void RC4(RC4_KEY *key, unsigned long len, const unsigned char *indata, unsigned char *outdata)
{
    lib_func[INDEX_RC4].func(key, len, indata, outdata);
}

int SSL_get_ex_data_X509_STORE_CTX_idx(void)
{
    return ssl_func[INDEX_SSL_get_ex_data_X509_STORE_CTX_idx].func();
}

void *X509_STORE_CTX_get_ex_data(X509_STORE_CTX *ctx,int idx)
{
    return (void *)lib_func[INDEX_X509_STORE_CTX_get_ex_data].func(ctx, idx);
}

int SSL_set_ex_data(SSL *ssl,int idx,void *data)
{
    return ssl_func[INDEX_SSL_set_ex_data].func(ssl, idx, data);
}

void *SSL_get_ex_data(const SSL *ssl,int idx)
{
    return (void *)ssl_func[INDEX_SSL_get_ex_data].func(ssl, idx);
}

int SSL_get_ex_new_index(long argl, void *argp, CRYPTO_EX_new *new_func,
			 CRYPTO_EX_dup *dup_func, CRYPTO_EX_free *free_func)
{
    return ssl_func[INDEX_SSL_get_ex_new_index].func(argl, argp, new_func, dup_func, free_func);
}

int SSL_CTX_set_default_verify_paths(SSL_CTX *ctx)
{
    return ssl_func[INDEX_SSL_CTX_set_default_verify_paths].func(ctx);
}

SSL_CTX * SSL_CTX_new(SSL_METHOD *meth)
{
    return (SSL_CTX *)ssl_func[INDEX_SSL_CTX_new].func(meth);
}

void  SSL_CTX_set_verify(SSL_CTX *ctx,int mode, int (*callback)(int, X509_STORE_CTX *))
{
    ssl_func[INDEX_SSL_CTX_set_verify].func(ctx, mode, callback);
}

long  SSL_CTX_ctrl(SSL_CTX *ctx,int cmd, long larg, void *parg)
{
    return (long)ssl_func[INDEX_SSL_CTX_ctrl].func(ctx, cmd, larg, parg);
}

SSL_METHOD * SSLv23_client_method(void)
{
    return (SSL_METHOD *)ssl_func[INDEX_SSLv23_client_method].func();
}

void  SSL_load_error_strings(void )
{
    ssl_func[INDEX_SSL_load_error_strings].func();
}

void  SSL_CTX_free(SSL_CTX *ctx)
{
    ssl_func[INDEX_SSL_CTX_free].func(ctx);
}

int  SSL_library_init(void)
{
    return ssl_func[INDEX_SSL_library_init].func();
}

void  SSL_free(SSL *ssl)
{
    ssl_func[INDEX_SSL_free].func(ssl);
}

int  SSL_shutdown(SSL *s)
{
    return ssl_func[INDEX_SSL_shutdown].func(s);
}

int  SSL_read(SSL *ssl,void *buf,int num)
{
    return ssl_func[INDEX_SSL_read].func(ssl, buf, num);
}

int  SSL_write(SSL *ssl,const void *buf,int num)
{
    return ssl_func[INDEX_SSL_write].func(ssl, buf, num);
}

#ifdef OPENSSL_0_9_8
int  SSL_want(const SSL *s)
#else
int  SSL_want(SSL *s)
#endif
{
    return ssl_func[INDEX_SSL_want].func(s);

}

#ifdef OPENSSL_0_9_8
int	SSL_get_error(const  SSL *s,int ret_code)
#else
int	SSL_get_error(SSL *s,int ret_code)
#endif
{
    return ssl_func[INDEX_SSL_get_error].func(s, ret_code);
}

#ifdef OPENSSL_0_9_8
const  char *SSL_get_version(const SSL *s)
#else
const  char *SSL_get_version(SSL *s)
#endif
{
    return (const char *)ssl_func[INDEX_SSL_get_version].func(s);
}

int  SSL_connect(SSL *ssl)
{
    return ssl_func[INDEX_SSL_connect].func(ssl);
}

#ifdef OPENSSL_0_9_8
void  SSL_copy_session_id(SSL *to,const SSL *from)
#else
void  SSL_copy_session_id(SSL *to,SSL *from)
#endif
{
    ssl_func[INDEX_SSL_copy_session_id].func(to, from);
}

long  SSL_ctrl(SSL *ssl,int cmd, long larg, void *parg)
{
    return (long)ssl_func[INDEX_SSL_ctrl].func(ssl, cmd, larg, parg);
}

int	SSL_set_fd(SSL *  s, int fd)
{
    return ssl_func[INDEX_SSL_set_fd].func(s, fd);
}

SSL * SSL_new(SSL_CTX *ctx)
{
    return (SSL *)ssl_func[INDEX_SSL_new].func(ctx);
}

ASN1_INTEGER *  X509_get_serialNumber(X509 *x)
{
    return (ASN1_INTEGER *)lib_func[INDEX_X509_get_serialNumber].func(x);
}

int	X509_cmp_current_time(ASN1_TIME *  s)
{
    return lib_func[INDEX_X509_cmp_current_time].func(s);
}

X509_NAME *  X509_get_issuer_name(X509 *a)
{
    return (X509_NAME *)lib_func[INDEX_X509_get_issuer_name].func(a);
}

EVP_PKEY *  X509_get_pubkey(X509 *x)
{
    return (EVP_PKEY *)lib_func[INDEX_X509_get_pubkey].func(x);
}

X509_NAME *  X509_get_subject_name(X509 *a)
{
    return (X509_NAME *)lib_func[INDEX_X509_get_subject_name].func(a);
}

int  X509_CRL_verify(X509_CRL *a, EVP_PKEY *r)
{
    return lib_func[INDEX_X509_CRL_verify].func(a, r);
}

void  X509_STORE_CTX_cleanup(X509_STORE_CTX *ctx)
{
    lib_func[INDEX_X509_STORE_CTX_cleanup].func(ctx);
}

int  X509_STORE_CTX_init(X509_STORE_CTX *ctx, X509_STORE *store,
			            X509 *x509, STACK_OF(X509) *chain)
{
    return lib_func[INDEX_X509_STORE_CTX_init].func(ctx, store, x509, chain);
}

void  X509_OBJECT_free_contents(X509_OBJECT *a)
{
    lib_func[INDEX_X509_OBJECT_free_contents].func(a);
}

int  X509_STORE_get_by_subject(X509_STORE_CTX *vs,int type,X509_NAME *name,
                              X509_OBJECT *ret)
{
    return lib_func[INDEX_X509_STORE_get_by_subject].func(vs, type, name, ret);
}

void  X509_STORE_CTX_set_error(X509_STORE_CTX *ctx,int s)
{
    lib_func[INDEX_X509_STORE_CTX_set_error].func(ctx, s);
}

X509 *  X509_STORE_CTX_get_current_cert(X509_STORE_CTX *ctx)
{
    return (X509 *)lib_func[INDEX_X509_STORE_CTX_get_current_cert].func(ctx);
}

char *  sk_value(const STACK * s, int a)
{
    return (char *)lib_func[INDEX_sk_value].func(s, a);
}

int  sk_num(const STACK *s)
{
    return lib_func[INDEX_sk_num].func(s);
}

char *  X509_NAME_oneline(X509_NAME *a,char *buf,int size)
{
    return (char *)lib_func[INDEX_X509_NAME_oneline].func(a, buf, size);
}

int  RAND_load_file(const char *file,long max_bytes)
{
    return lib_func[INDEX_RAND_load_file].func(file, max_bytes);
}

int  RAND_status(void)
{
    return lib_func[INDEX_RAND_status].func();
}

int  RAND_egd(const char *path)
{
    return lib_func[INDEX_RAND_egd].func(path);
}

const  char *RAND_file_name(char *file,size_t num)
{
    return (const char *)lib_func[INDEX_RAND_file_name].func(file, num);
}

int  RAND_write_file(const char *file)
{
    return lib_func[INDEX_RAND_write_file].func(file);
}

int  BIO_sock_should_retry(int i)
{
    return lib_func[INDEX_BIO_sock_should_retry].func(i);
}

long  ASN1_INTEGER_get(ASN1_INTEGER *a)
{
    return (long)lib_func[INDEX_ASN1_INTEGER_get].func(a);
}

int  ASN1_INTEGER_cmp(ASN1_INTEGER *x, ASN1_INTEGER *y)
{
    return lib_func[INDEX_ASN1_INTEGER_cmp].func(x, y);
}

char *  ERR_error_string(unsigned long e,char *buf)
{
    return (char *)lib_func[INDEX_ERR_error_string].func(e, buf);
}

void  ERR_print_errors_fp(FILE *fp)
{
    lib_func[INDEX_ERR_print_errors_fp].func(fp);
}

static const char *ssl_version(unsigned long version)
{
    static char str[25];
    snprintf(str, 10, "%lu.%lu.%lu",
             (version & 0xF0000000L) >> (7 * 4),
             (version & 0x0FF00000L) >> (5 * 4),
             (version & 0x000FF000L) >> (3 * 4));
    if ((version & 0x0000000FL) == 0xf) {
        if (((version & 0x00000FF0L) >> (1 * 4)) > 0) {
            char tmp[2];
            tmp[0] = 'a' + (char)(((version & 0x00000FF0L) >> (1 * 4)) - 1);
            strcat(str, tmp);        
        } else {
            /* Nothing to add */
        }
    } else {
        if ((version & 0x00000FFFL) > 0) {
            char tmp[10];
            snprintf(tmp, 10, "-beta%lu", (version & 0x00000FFFL));
            strcat(str, tmp);
        } else {
            strcat(str, "-dev");
        } 
    }

    return str;
}

BOOL initSSLWrapper(HMODULE ssl_dll, HMODULE lib_dll)
{
    DLL_FUNC SSLeay_func;
    ssl_func_t *cur;
    cur = ssl_func;

    if ((SSLeay_func = (DLL_FUNC)GetProcAddress(lib_dll, "SSLeay"))) {
        unsigned long version;
        version = (unsigned long)SSLeay_func();

        if (version < 0x00907000L || version >= 0x00909000L) {
            fprintf(stdout, "PFTP: Warning, this version of PFTP was built with version 0.9.7 and 0.9.8 of OpenSSL in mind.\n");
        }

        fprintf(stdout, "PFTP: Using OpenSSL library version %s.\n",
                ssl_version(version));
    } else {
        fprintf(stderr, "PFTP: Unable to load function `%s' from libeay32.dll (%lu).\n",
                "SSLeay", GetLastError());
        return FALSE;
    }
    
    while (cur->name) {
        if (!(cur->func = (DLL_FUNC)GetProcAddress(ssl_dll, cur->name))) {
            fprintf(stderr, "PFTP: Unable to load function `%s' from ssleay32.dll (%lu).\n",
                    cur->name, GetLastError());
            return FALSE;
        }
        cur++;
    }
#ifdef _DEBUG
    fprintf(stderr, "PFTP: Loaded %lu functions from ssleay32.dll.\n",
	    sizeof(ssl_func) / sizeof(ssl_func_t));
#endif

    cur = lib_func;

    while (cur->name) {
        if (!(cur->func = (DLL_FUNC)GetProcAddress(lib_dll, cur->name))) {
            fprintf(stderr, "PFTP: Unable to load function `%s' from libeay32.dll (%lu).\n",
                    cur->name, GetLastError());
            return FALSE;
        }
        cur++;
    }        
#ifdef _DEBUG
    fprintf(stderr, "PFTP: Loaded %lu functions from libeay32.dll.\n",
	    sizeof(lib_func) / sizeof(ssl_func_t));
#endif

    return TRUE;
}

#endif /* NO_SSL */